/*
    dp : An implementation of the davis puttnam procedure for solving
         propositional satisfiability problems.
    Copyright (C) 1993  Peter Barth

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ------------------------------------------------------------------
   PBLIST.C
* -------------------------------------------------------------------
* Auth: pb
* Purp: double linked lists
* Hist:==============================================================
* Date         Name    What has been done ?!
* -------------------------------------------------------------------
* --> PBLIST.H
* ---------------------------------------------------------------- */

#include "stdio.h"
#include "stdlib.h"
#include "pblist.h"

char *l_errstring;
int l_errno = 0;

char *l_errtab[] = {
  "ok",
  "no more memory",
  "problem writing data",
  "problem reading data",
  NULL
};

list l_first(list l)
{
  if (l != NULL) 
    { 
      while (l->prev != NULL) 
	l = l->prev; 
      l_seterror(0); 
    } 
  return l;
}

list l_last(list l)
{ 
  if (l != NULL) 
    { 
      while (l->next != NULL) 
	l = l->next; 
      l_seterror(0); 
    } 
  return l; 
}

list l_ith(list l,int i)
{ 
  if (i > 1) 
  { 
    while ((i-- > 1) && (l != NULL)) 
      l = l->next; 
    l_seterror(0); 
  } 
  return l; 
}

void * l_dith(list l,int i)
{ 
   l = l_ith(l,i); 
   return (l == NULL) ? NULL : l->data; 
}

void * l_rdata(list where)
{ 
   return (where == NULL) ? NULL : where->data; 
}

list l_wdata(list where,void *  datum) 
{ 
   if (where != NULL) 
     where->data = datum; 
   return where; 
}

int l_len(list l)
{ 
  int len = 0; 
  while (l!=NULL) 
    { 
      len++;
      l = l->next; 
    } 
  l_seterror(0); 
  return len; 
}



/************ konstruktive Funktionen *************/

list l_create()
{ list l = (list) malloc(sizeof(struct _List));
  if (l == NULL) 
    l_seterror(1);
  else
  {
    l->prev = l->data = l->next = NULL;
    l_seterror(0);
  }
  return (l);
}

list l_dcreate(void *data)
{
  list l = l_create();

  if (!l_errno)
    l->data = data;
  return(l);
}

list l_insb(list where,list what)
{
  list lw = l_last(what);

  if (where == NULL)
    return (what);
  if (what == NULL)
    return (where);
  what->prev = where->prev;
  lw->next = where;
  if (where->prev != NULL)
    (where->prev)->next = what;
  where->prev = lw;
  l_seterror(0);
  return (what);
}

list l_dinsb(list where,void * data) 
{ 
  list l = l_dcreate(data); 
  if (!l_errno) 
    where = l_insb(where,l); 
  return where; 
}

list l_insa(list where,list what)
{
  if (where == NULL)
    return (what);
  if (what == NULL)
    return (where);
  if (where->next != NULL)
    return (l_insb(where->next,what));
  where->next = what;
  what->prev = where;
  return(what);
}

list l_dinsa(list where,void * data) 
{ 
  list l = l_dcreate(data); 
  if (!l_errno) 
    where = l_insa(where,l); 
  return where; 
}


/* destructive functions */

list l_del(list what) 
{
  list l = NULL; 
  if (what != NULL) 
    { 
      if (what->next != NULL) 
	(what->next)->prev = what->prev; 
      if (what->prev != NULL) 
	(what->prev)->next = what->next; 
      l = (what->next != NULL) ? what->next : what->prev; 
    free(what); 
  } 
  return l; 
}

list l_delall(list where)
{
  list l;

  if (where == NULL)
    return(NULL);
  l = where->prev;
  while ((where = l_del(where)) != NULL);
  return(l);
}

/************ datensatzbezogene Funktionen *************/


void l_forall(list l,void (* f)(void *data))
{
  while (l != NULL)
  {
    (* f)(l->data);
    l = l->next;
  }
  l_seterror(0);
}

list l_search(list l,void *data,int (* equal)(void *data,void *dptr))
{
  while ((l != NULL) && (!((*equal) (l->data,data))))
    l = l->next;
  l_seterror(0);
  return (l);
}


/* The one and only : BUBBLE (I have not need it yet) */
list l_sort(list l,int (* comp)(void *dptr1,void *dptr2))
{
  list l1,l2;
  void *ptr;
  int i,j,len;

  if (l == NULL)
    return(NULL);
  len = l_len(l);
  for (i=1;i<=len;i++)
  {
    l1 = l;
    l2 = l->next;
    for (j=1;j<=len-1;j++)
    {
      if ((*comp)(l1->data,l2->data) > 0)
      {
        ptr = l1->data;
        l1->data = l2->data;
        l2->data = ptr;
      }
      l1 = l2;
      l2 = l2->next;
    }
  }
  l_seterror(0);
  return (l);
}

/* some prototypes */
size_t fwrite(const void*, size_t, size_t, FILE*);
int    rewind(FILE*);
int    fclose(FILE*);
size_t fread(void*, size_t, size_t, FILE*);



list l_save(list l,size_t len,char *dateiname)
{
  FILE *datei;
  list l1 = l;

  l_seterror(0);
  if (l == NULL)
    return (NULL);
  if ((datei = fopen (dateiname,"wb")) != NULL)
  {
    while (l != NULL)
    {
      if (fwrite (l->data,len,1,datei) != 1)
      {
        l_seterror(2);
        rewind(datei);
        fclose(datei);
        return(l1);
      }
      l = l->next;
    }
    fclose (datei);
    return (l1);
  }
  l_seterror(2);
  return (l1);
}

list l_load(size_t len,char *dateiname)
{
  FILE *datei;
  list l = NULL;
  void *data;

  if ((datei = fopen (dateiname,"rb")) != NULL)
  {
    do
    {
      data = malloc(len);
      if (data == NULL)
      {
        fclose (datei);
        l = l_first(l);
        l_forall(l,free);
        l_seterror(1);
        return (l_delall(l));
      }
      fread(data,len,1,datei);
      if (!feof(datei))
        l = l_dinsa(l,data);
      if (l_errno || ferror(datei))
      {
        fclose (datei);
        l = l_first(l);
        l_forall(l,free);
        if (!l_errno)
          l_seterror(3);
        return (l_delall(l));
      }
    } while (!feof(datei));
    fclose (datei);
    return (l_first(l));
  }
  l_seterror(3);
  return(NULL);
}
