/*
    dp : An implementation of the davis puttnam procedure for solving
         propositional satisfiability problems.
    Copyright (C) 1993  Peter Barth

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ------------------------------------------------------------------
   PBLIST.H
* -------------------------------------------------------------------
* Auth: pb
* Purp: double linked lists
* Hist:==============================================================
* Date         Name    What has been done ?!
* -------------------------------------------------------------------
* 6.6. 1989    pb      creation
* 4.11.1989    pb      more standard C
* 6.11.1990    pb      english (standard l_op)
* 7.11.1990    pb      more save
* ---------------------------------------------------------------- */

#ifndef __PBLIST
#define __PBLIST

#include "stddef.h"

typedef struct _List {
  struct _List *prev;
  void         *data;
  struct _List *next;
} *list;

extern char *l_errstring;
extern int l_errno;
extern char *l_errtab[];

#define l_seterror(errno)  l_errstring = l_errtab[l_errno = errno] 


/* navigating and info functions */

#define l_next(l) ((list) (l == NULL) ? NULL : l->next)
#define l_prev(l) ((list) (l == NULL) ? NULL : l->prev)
list l_first(list l);
list l_last(list l);

list l_ith(list l,int i);
void * l_dith(list l,int i);
void * l_rdata(list where);
list l_wdata(list where,void * datum);

int l_len(list l);


/* constructive functions */

list l_create(void);
list l_dcreate(void *data);

list l_insb(list where,list what);
list l_dinsb(list where,void * data);

list l_insa(list where,list what);
list l_dinsa(list where,void * data);


/* destructive functions */

list l_del(list what);
list l_delall(list l);


/* some other usefull functions */

void l_forall(list l,void (* f)(void *data));
list l_search(list l,void *data,int (* equal)(void *data,void *dptr));
list l_sort(list l,int (* comp)(void *dptr1,void *dptr2));

list l_save(list l,size_t len,char *dateiname);
/* save complete list into file
 * saved information:  *dptr .. *(dptr+size_t)
 * Attention !!!!: not usefull if data conatins pointers
 */

list l_load(size_t len,char *dateiname);
/* load saved list
 * Attention !!!!: allocates Room for data (len)
 */

#endif





